﻿using Microsoft.EntityFrameworkCore;
using SushiBarContracts.BindingModels;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDatabaseImplement.Models;
using SushiBarDataModels.Enums;

namespace SushiBarDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new SushiBarDatabase();

            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);

            if (element == null) return null;
            context.Orders.Remove(element);
            context.SaveChanges();

            return element.GetViewModel;

        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            using var context = new SushiBarDatabase();

            return context.Orders
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .FirstOrDefault(x =>
                        (model.Status == null || model.Status != null && model.Status.Contains(x.Status)) &&
                        model.ImplementerId.HasValue && x.ImplementerId == model.ImplementerId ||
                        model.Id.HasValue && x.Id == model.Id
                    )
                    ?.GetViewModel;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel? model)
        {
            if (model.Id.HasValue)
            {
                var result = GetElement(model);
                return result != null ? new() { result } : new();
            }

            using var context = new SushiBarDatabase();
            IQueryable<Order>? queryWhere = null;

            if (model.DateFrom.HasValue && model.DateTo.HasValue)
            {
                queryWhere = context.Orders
                    .Where(x => model.DateFrom <= x.DateCreate.Date &&
                                                  x.DateCreate.Date <= model.DateTo);
            }
            else if (model.Status != null)
            {
                queryWhere = context.Orders
                    .Where(x => model.Status.Contains(x.Status));
            }
            else if (model.ClientId.HasValue)
            {
                queryWhere = context.Orders
                    .Where(x => x.ClientId == model.ClientId);
            }
            else
            {
                return new();
            }
            return queryWhere
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new SushiBarDatabase();

            return context.Orders
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);

            if (newOrder == null)
            {
                return null;
            }

            using var context = new SushiBarDatabase();

            context.Orders.Add(newOrder);
            context.SaveChanges();

            return newOrder.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new SushiBarDatabase();

            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);

            if (order == null)
            {
                return null;
            }

            order.Update(model);
            context.SaveChanges();

            return order.GetViewModel;
        }
    }
}
