﻿using SushiBarFileImplement.Models;
using System.Xml.Linq;

namespace SushiBarFileImplement
{
    public class DataFileSingleton
    {
        private static DataFileSingleton? instance;
        private readonly string ComponentFileName = "Component.xml";
        private readonly string OrderFileName = "Order.xml";
        private readonly string SushiFileName = "Sushi.xml";
        private readonly string ImplementerFileName = "Implementer.xml";
        public List<Component> Components { get; private set; }
        public List<Order> Orders { get; private set; }
        public List<Sushi> Sushis { get; private set; }
        public List<Implementer> Implementers { get; set; }
        public static DataFileSingleton GetInstance()
        {
            instance ??= new DataFileSingleton();
            return instance;
        }
        public void SaveComponents() => SaveData(Components, ComponentFileName, "Components", x => x.GetXElement);
        public void SaveSushis() => SaveData(Sushis, SushiFileName, "Sushis", x => x.GetXElement);
        public void SaveOrders() => SaveData(Orders, OrderFileName, "Orders", x => x.GetXElement);
        public void SaveImplementers() => SaveData(Implementers, ImplementerFileName, "Implementers", x => x.GetXElement);
        
        private DataFileSingleton()
        {
            Components  = LoadData(ComponentFileName, "Component", x => Component.Create(x)!)!;
            Sushis      = LoadData(SushiFileName, "Sushi", x => Sushi.Create(x)!)!;
            Orders      = LoadData(OrderFileName, "Order", x => Order.Create(x)!)!;
            Implementers = LoadData(ImplementerFileName, "Implementer", x => Implementer.Create(x)!)!;
        }
        private static List<T>? LoadData<T>(string filename, string xmlNodeName, Func<XElement, T> selectFunction)
        {
            if (File.Exists(filename))
            {
                return XDocument.Load(filename)?.Root?.Elements(xmlNodeName)?.Select(selectFunction)?.ToList();
            }
            return new List<T>();
        }
        private static void SaveData<T>(List<T> data, string filename, string xmlNodeName, Func<T, XElement> selectFunction)
        {
            if (data != null)
            {
                new XDocument(new XElement(xmlNodeName, data.Select(selectFunction).ToArray())).Save(filename);
            }
        }
    }
}
