﻿using SushiBarContracts.BindingModels;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Enums;
using SushiBarDataModels.Models;
using System.Xml.Linq;

namespace SushiBarFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public string SushiName { get; private set; } = string.Empty;
        public int SushiId { get; private set; }
        public int ClientId { get; }
        public int? ImplementerId { get; set; }
        public int Count { get; private set; }
        public double Sum { get; private set; }
        public OrderStatus Status { get; private set; } = OrderStatus.Unknown;
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; set; }

        public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                SushiId = model.SushiId,
                SushiName = model.SushiName,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }

            var order = new Order()
            {
                Id         = Convert.ToInt32(element.Attribute("Id")!.Value),
                SushiId    = Convert.ToInt32(element.Element("SushiId")!.Value),
                SushiName  = element.Element("SushiName")!.Value,
                Count      = Convert.ToInt32(element.Element("Count")!.Value),
                Sum        = Convert.ToDouble(element.Element("Sum")!.Value),
                Status     = (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value),
                ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
                DateCreate = DateTime.ParseExact(element.Element("DateCreate")!.Value, "G", null)
            };

            DateTime.TryParse(element.Element("DateImplement")!.Value, out DateTime dateImpl);
            order.DateImplement = dateImpl;

            return order;
        }
        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            SushiId = model.SushiId;
            SushiName = model.SushiName;
            ImplementerId = model.ImplementerId;
            Count = model.Count;
            Sum = model.Sum;
            Status = model.Status;
            DateCreate = model.DateCreate;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            SushiId = SushiId,
            SushiName = SushiName,
            ImplementerFio = DataFileSingleton.GetInstance()
                .Implementers
                .FirstOrDefault(x => x.Id == ImplementerId)?.ImplementerFio ?? string.Empty,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };

        public XElement GetXElement => new("Order",
            new XAttribute("Id", Id),
            new XElement("SushiName", SushiName),
            new XElement("SushiId", SushiId.ToString()),
            new XElement("ImplementerId", ImplementerId),
            new XElement("Count", Count.ToString()),
            new XElement("Sum", Sum.ToString()),
            new XElement("Status", Status.ToString()),
            new XElement("DateCreate", DateCreate.ToString()),
            new XElement("DateImplement", DateImplement.ToString()));
    }
}
