﻿using Microsoft.AspNetCore.Mvc;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModels;
using SushiBarContracts.ViewModels;

namespace SushiBarRestApi.Controllers;

[Route("api/[controller]/[action]")]
[ApiController]
public class MainController : Controller
{
    private readonly ILogger _logger;
    private readonly IOrderLogic _order;
    private readonly ISushiLogic _sushi;
    public MainController(ILogger<MainController> logger, IOrderLogic order, ISushiLogic sushi)
    {
        _logger = logger;
        _order = order;
        _sushi = sushi;
    }
    [HttpGet]
    public List<SushiViewModel>? GetProductList()
    {
        try
        {
            return _sushi.ReadList(null);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Error on getting list of sushi");
            throw;
        }
    }
    [HttpGet]
    public SushiViewModel? GetProduct(int sushiId)
    {
        try
        {
            return _sushi.ReadElement(new SushiSearchModel() { Id = sushiId });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Error getting sushi with id={Id}", sushiId);
            throw;
        }
    }
    [HttpGet]
    public List<OrderViewModel>? GetOrders(int clientId)
    {
        try
        {
            return _order.ReadList(new OrderSearchModel { ClientId = clientId });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Error getting orders for client with id={Id}", clientId);
            throw;
        }
    }
    [HttpPost]
    public void CreateOrder(OrderBindingModel model)
    {
        try
        {
            _order.CreateOrder(model);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Error create order");
            throw;
        }
    }
}