﻿using SushiBarContracts.BindingModels;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Enums;
using SushiBarDataModels.Models;
using System.Reflection;

namespace SushibarListImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public string SushiName { get; private set; } = string.Empty;
        public int SushiId { get; private set; }
        public int ClientId { get; }
        public int Count { get; private set; }
        public double Sum { get; private set; }
        public OrderStatus Status { get; set; } = OrderStatus.Unknown;
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; set; } = null;

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                SushiId = model.SushiId,
                SushiName = model.SushiName,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Id = model.Id;
            SushiId = model.SushiId;
            SushiName = model.SushiName;
            Count = model.Count;
            Sum = model.Sum;
            Status = model.Status;
            DateCreate = model.DateCreate;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            SushiId = SushiId,
            SushiName = SushiName,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
